#include <string>
#include "xlib.h"
#include <map>
#include "Font.h"

#include <sigc++/sigc++.h> // zomg signals

#define CONFIG_FILE "/.config/visibility/config"
#define OVERRIDES_FILE "/.config/visibility/overrides"

namespace Visibility {

class Config {
	public:
		static Config * instance(void);
		void set_config_file(std::string file);

		void font_setup(void);

		unsigned image_size;
		unsigned spacing;
		unsigned desktop_separation;
		unsigned tooltip_padding;
		unsigned text_spacing;

		double tooltip_time;

		bool show_desktop_names;
		bool single_desktop_mode;
		bool set_window_type;

		// main window stuff

		unsigned border_width;
		CARD32 border_colour;
		CARD32 bg_colour;

		CARD32 active_text_colour, inactive_text_colour;

		CARD32 active_bg_colour, inactive_bg_colour;
		CARD32 tooltip_bg_colour;
		CARD32 tooltip_text_colour;
		CARD32 tooltip_border_colour;

		Font *font, *tooltip_font;

		std::string orientation;

		unsigned gap_x, gap_y;
		bool set_partial_strut;

		// tint settings
		CARD32 active_tint, inactive_tint, iconified_tint;
		double active_tint_amount, inactive_tint_amount, iconified_tint_amount;

		void reload(void);

		std::string get_icon_override(std::string& name);

		sigc::signal<void> signal_changed; // reload() emits this
	protected:
		Config(void);
		~Config();
		Config(const Config&);
		Config& operator= (const Config&);

		std::string const home_path;

		std::map<std::string, std::string> icon_overrides;
		std::string config_file;

		std::string font_name;
		std::string tooltip_font_name;

		void parse_file(std::string path);
		void parse_icon_overrides(std::string path);
		void set_defaults(void);

	private:
		static Config * pinstance;
};

}
