// a key-value config file parser.

#ifndef CONFIG_PARSER_HEADER_INCLUDED
#define CONFIG_PARSER_HEADER_INCLUDED

#include <string>
#include <map>
#include <sstream>
#include <iostream>

class ConfigParser {
	public:
		// if file_name is specified, parses that file
		ConfigParser(std::string file_name = "");
		~ConfigParser();

		// parses this file and modifies 'values' as necessary. this is especially
		// useful for reading in one config file and then overriding values using
		// another config file.
		// returns false if it could not open the file.
		bool parse(std::string file_name);

		// returns true if the config file defines this key
		inline bool has_key(std::string key) const {
			if (values.find(key) != values.end()) return true;
			return false;
		}

		// self explanatory

		inline std::string get_value_string(std::string const & key) {
			return values[key];
		}

		int get_value_int(std::string const & key);
		double get_value_double(std::string const & key);
		bool get_value_bool(std::string const & key);

	protected:
		// maps key -> value in string format
		std::map<std::string, std::string> values;
		std::string file_name;
};

#endif
