#include "Server.h"
#include <iostream>

namespace Visibility {

sigc::signal<void> Server::signal_desktop_changed;
sigc::signal<void> Server::signal_desktop_names_changed;
sigc::signal<void> Server::signal_number_of_desktops_changed;

// initialize the sekrit pointer
Server * Server::pinstance = 0;
Server * Server::instance(void) {
	if (!pinstance) {
		pinstance = new Server;
	}
	return pinstance;
}

Server::Server(void) {
	display = XOpenDisplay(NULL);
	default_screen = DefaultScreen(display);
	root_window = RootWindow(display, default_screen);
	visual = DefaultVisual(display, default_screen);
	depth = DefaultDepth(display, default_screen);
	default_gc = DefaultGC(display, default_screen);
	colormap = DefaultColormap(display, default_screen);

	// set up atoms
	net_active_window_atom = XInternAtom(display, "_NET_ACTIVE_WINDOW", False);
	net_current_desktop_atom = XInternAtom(display, "_NET_CURRENT_DESKTOP", False);
	net_number_of_desktops_atom = XInternAtom(display, "_NET_NUMBER_OF_DESKTOPS", False);
	net_desktop_names_atom = XInternAtom(display, "_NET_DESKTOP_NAMES", False);
	net_client_list_atom = XInternAtom(display, "_NET_CLIENT_LIST", False);
	net_wm_icon_name_atom = XInternAtom(display, "_NET_WM_ICON_NAME", False);
	utf8_string_type = XInternAtom(display, "UTF8_STRING", False);
	net_wm_icon_atom = XInternAtom(display, "_NET_WM_ICON", False);
	wm_hints_atom = XInternAtom(display, "WM_HINTS", False);
	net_wm_desktop_atom = XInternAtom(display, "_NET_WM_DESKTOP", False);
	net_wm_state_atom = XInternAtom(display, "_NET_WM_STATE", False);
	net_wm_state_skip_pager_atom = XInternAtom(display, "_NET_WM_STATE_SKIP_PAGER",
		False);
	net_wm_state_skip_taskbar_atom = XInternAtom(display, "_NET_WM_STATE_SKIP_TASKBAR",
		False);
	net_wm_state_sticky_atom = XInternAtom(display, "_NET_WM_STATE_STICKY",
		False);
	net_wm_state_hidden_atom = XInternAtom(display, "_NET_WM_STATE_HIDDEN",
		False);
	net_wm_state_shaded_atom = XInternAtom(display, "_NET_WM_STATE_SHADED",
		False);
	net_wm_state_attention_atom = XInternAtom(display,
		"_NET_WM_STATE_DEMANDS_ATTENTION", False);

	wm_name_atom = XInternAtom(display, "WM_NAME", False);

	net_frame_extents_atom = XInternAtom(display, "_NET_FRAME_EXTENTS", False);

	net_wm_window_type_atom = XInternAtom(display, "_NET_WM_WINDOW_TYPE", False);
	net_wm_window_type_desktop_atom = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DESKTOP", False);
	net_wm_window_type_dock_atom = XInternAtom(display, "_NET_WM_WINDOW_TYPE_DOCK", False);
	net_wm_window_type_toolbar_atom = XInternAtom(display, "_NET_WM_WINDOW_TYPE_TOOLBAR", False);
	net_wm_window_type_menu_atom = XInternAtom(display, "_NET_WM_WINDOW_TYPE_MENU", False);
	net_wm_window_type_splash_atom = XInternAtom(display, "_NET_WM_WINDOW_TYPE_SPLASH", False);
}


Server::~Server() {
	XCloseDisplay(display);
	XFree(display);
}

void Server::get_client_list(std::vector< ::Window >& client_list) {
	Atom actual_type_return;
	int actual_format_return = 0;
	unsigned long nitems_return = 0;
	unsigned long bytes_after_return = 0;
	::Window * windows = NULL;

	if (XGetWindowProperty(display, root_window, net_client_list_atom, 0, 0x7fffffff, False, XA_WINDOW, &actual_type_return, &actual_format_return, &nitems_return, &bytes_after_return, (unsigned char **)&windows) != Success) {
		std::cout << "failed to get window list." << std::endl;
		return;
	}

	// it worked

	for (unsigned long count = 0; count < nitems_return; count++) {
		if (windows[count] == 0) {
			std::cout << "wtf?" << std::endl;
		} else {
			client_list.push_back(windows[count]);
		}
	}

	XFree(windows);
}

CARD32 Server::get_current_desktop(void) {
	Atom actual_type_return;
	int actual_format_return = 0;
	unsigned long nitems_return = 0;
	unsigned long bytes_after_return = 0;
	// not long because long is 64bit on some systems parhaps
	CARD32 * desktop = 0;

	if (XGetWindowProperty(display, root_window, net_current_desktop_atom, 0, 1,
		False, XA_CARDINAL, &actual_type_return, &actual_format_return,
		&nitems_return, &bytes_after_return, (unsigned char**)&desktop) != Success) {
			return 0;
	}
	if (actual_type_return != XA_CARDINAL || nitems_return == 0) {
		return 0;
	}

	unsigned ret_val = desktop[0];
	XFree(desktop);

	return ret_val;
}

CARD32 Server::get_number_of_desktops(void) {
	Atom actual_type_return;
	int actual_format_return = 0;
	unsigned long nitems_return = 0;
	unsigned long bytes_after_return = 0;
	// not long because long is 64bit on some systems parhaps
	CARD32 * desktops = 0;

	if (XGetWindowProperty(display, root_window, net_number_of_desktops_atom, 0, 1,
		False, XA_CARDINAL, &actual_type_return, &actual_format_return,
		&nitems_return, &bytes_after_return, (unsigned char**)&desktops) != Success) {
			return 0;
	}
	if (actual_type_return != XA_CARDINAL || nitems_return == 0) {
		return 0;
	}

	unsigned ret_val = desktops[0];
	XFree(desktops);
	return ret_val;
}

static void desktop_list_to_vector(char * desktops, unsigned length,
	std::vector<std::string>& names) {

	unsigned pos = 0;
	while (pos < length) {
		std::string string;
		// make the string
		char c;
		while ((c = desktops[pos++]) != '\0') {
			string += c;
		}
		// the string is made.
		names.push_back(string);
	}
}

void Server::get_desktop_names(std::vector<std::string>& names) {
	Atom actual_type_return;
	int actual_format_return = 0;
	unsigned long nitems_return = 0;
	unsigned long bytes_after_return = 0;
	// not long because long is 64bit on some systems parhaps
	char * desktops = 0;

	int result;

	result = XGetWindowProperty(display, root_window, net_desktop_names_atom, 0, 100,
		False, utf8_string_type, &actual_type_return, &actual_format_return,
		&nitems_return, &bytes_after_return, (unsigned char**)&desktops);

	if (result != Success || actual_type_return != utf8_string_type) return;

	/*for(unsigned count = 0; count < nitems_return; count++) {
		printf("%c\n", desktops[count]);
	}*/

	desktop_list_to_vector(desktops, nitems_return, names);

	XFree(desktops);
}

void Server::get_display_size(unsigned& width, unsigned& height) {
	// someone will have a seizure if either of these is negative
	width = (unsigned)DisplayWidth(display, default_screen);
	height = (unsigned)DisplayHeight(display, default_screen);
}

bool Server::set_current_desktop(CARD32 desktop) {
	XEvent event;
	long mask = SubstructureRedirectMask | SubstructureNotifyMask;

	event.xclient.type = ClientMessage;
	event.xclient.serial = 0;
	event.xclient.send_event = True;
	event.xclient.display = display;
	event.xclient.window = root_window;
	event.xclient.message_type = net_current_desktop_atom;

	event.xclient.format = 32;
	event.xclient.data.l[0] = desktop;
	event.xclient.data.l[1] = 0;
	event.xclient.data.l[2] = 0;
	event.xclient.data.l[3] = 0;
	event.xclient.data.l[4] = 0;

	if (XSendEvent(display, root_window, False, mask, &event)) {
		return true;
	}
	return false;
}

bool Server::flush(void) {
	if (XFlush(display) == Success) {
		return true;
	}
	return false;
}

}
