// a singleton to handle important x-server related stuff.

#ifndef SERVER_HEADER_INCLUDED
#define SERVER_HEADER_INCLUDED

#include "Window.h"
#include "xlib.h"

// STL ftw
#include <vector>
#include <string> // not stl, but still ftw.

#include <sigc++/sigc++.h> // zomg signals

namespace Visibility {

class Server {
	public:
		// signals!
		static sigc::signal<void> signal_desktop_changed; // emitted when the current desktop changes
		static sigc::signal<void> signal_desktop_names_changed;
		static sigc::signal<void> signal_number_of_desktops_changed;

		static Server * instance(void);

		void get_client_list(std::vector< ::Window >& client_list);
		CARD32 get_number_of_desktops(void);
		void get_desktop_names(std::vector<std::string>& names);
		CARD32 get_current_desktop(void);
		bool set_current_desktop(CARD32 desktop);

		bool flush(void);

		void get_display_size(unsigned& width, unsigned& height);

		Atom net_active_window_atom;
		Atom net_current_desktop_atom;
		Atom net_number_of_desktops_atom;
		Atom net_desktop_names_atom;
		Atom net_client_list_atom;
		Atom net_wm_icon_name_atom;
		Atom utf8_string_type;
		Atom net_wm_icon_atom;
		Atom wm_hints_atom;
		Atom net_wm_desktop_atom;
		Atom net_frame_extents_atom;
		Atom wm_name_atom;

		// window state atoms

		Atom net_wm_state_atom;
		Atom net_wm_state_skip_pager_atom;
		Atom net_wm_state_skip_taskbar_atom;
		Atom net_wm_state_sticky_atom;
		Atom net_wm_state_hidden_atom;
		Atom net_wm_state_shaded_atom;
		Atom net_wm_state_attention_atom;

		Atom net_wm_window_type_atom;

		// window type atoms
		Atom net_wm_window_type_desktop_atom;
		Atom net_wm_window_type_dock_atom;
		Atom net_wm_window_type_toolbar_atom;
		Atom net_wm_window_type_menu_atom;
		Atom net_wm_window_type_splash_atom;


		// this is wrong.
		// don't do this yourself.
		Display * display;
		::Window root_window;
		int default_screen;
		Visual * visual;
		int depth;
		GC default_gc;
		Colormap colormap;

	protected:
		Server(void);
		~Server();
		Server(const Server&);
		Server& operator= (const Server&);

	private:
		static Server * pinstance;
};

}

#endif
