#include "Tooltip.h"
#include "Server.h"
#include "Config.h"
#include <iostream>

namespace Visibility {

unsigned Tooltip::height = 0;

Tooltip::Tooltip(std::string text_) {
	Server * server = Server::instance();
	XSetWindowAttributes attributes;
	attributes.override_redirect = True;
	::Window window_ = XCreateWindow(server->display, server->root_window, 0, 0,
		10, 10, 0, server->depth, InputOutput, server->visual, CWOverrideRedirect,
		&attributes);

	this->window = Window::get_window(window_);
	if (text_.length()) set_text(text_);
}

Tooltip::~Tooltip() {
	window->unmap();
	window->destroy();
}

void Tooltip::set_text(std::string text_) {
	text = text_;
	Font::TextMetrics metrics;
	Config * config = Config::instance();
	config->tooltip_font->get_metrics(text, metrics);
	width = metrics.extents.width + 2 * config->tooltip_padding + 2; // 2px bw

	height = metrics.height + 2 * config->tooltip_padding + 2;
	unsigned baseline = metrics.calculate_baseline(height);

	std::tr1::shared_ptr<Pixmap> pixmap(new Pixmap(window->drawable, width, height));
	pixmap->fill_rectangle(config->tooltip_border_colour, 0, 0, width, height);
	pixmap->fill_rectangle(config->tooltip_bg_colour, 1, 1, width - 2, height - 2);

	pixmap->draw_text((0xff << 24) | config->tooltip_text_colour,
		config->tooltip_font, text,
		metrics.extents.x + 1 + config->tooltip_padding, baseline);

	window->resize(width, height);
	window->set_background_pixmap(pixmap);
	window->clear();
}

void Tooltip::move(unsigned x, unsigned y) {
	window->move(x, y);
}

void Tooltip::map(void) {
	window->map();
}

void Tooltip::unmap(void) {
	window->unmap();
}

}
