#ifndef WINDOW_HEADER_INCLUDED
#define WINDOW_HEADER_INCLUDED

#include "xlib.h"
#include "ARGBData.h"
#include <vector>

#include "Drawable.h"
#include "Pixmap.h"

#include <sigc++/sigc++.h> // zomg signals
#include <map>

#include <tr1/memory>

namespace Visibility {

	class Window : public Drawable {
	protected:
		static std::map< ::Window, std::tr1::shared_ptr<Window> > windows;


	public:
		// to wrap an existing window
		Window(::Window window);

		// a new window
		Window(unsigned x, unsigned y, unsigned width, unsigned height, ::Window parent = 0);

		~Window();
		static std::tr1::shared_ptr<Window> get_window(::Window window);

		// make new windows
		static std::tr1::shared_ptr<Window> get_window(unsigned x, unsigned y, unsigned width, unsigned height, ::Window parent = 0);

		// signals! they are self explanatory, yell at me if they aren't.

		sigc::signal<void, ::Window> signal_closed;
		sigc::signal<void, ::Window> signal_icon_changed;
		sigc::signal<void, ::Window> signal_title_changed;
		sigc::signal<void, ::Window> signal_state_changed;
		sigc::signal<void, ::Window> signal_type_changed;
		// cool beans
		sigc::signal<void, ::Window> signal_pointer_entered;
		sigc::signal<void, ::Window> signal_pointer_left;
		sigc::signal<void, ::Window> signal_clicked;

		static sigc::signal<void> signal_active_window_changed;
		static sigc::signal<void, ::Window> signal_new;
		static sigc::signal<void, ::Window> signal_window_desktop_changed;

		// things you can do to windows

		bool resize(unsigned width, unsigned height);
		bool move(unsigned x, unsigned y);

		void get_position(unsigned& x, unsigned& y);
		bool get_frame_extents(CARD32& left, CARD32& right, CARD32& top,
													 CARD32& bottom);

		void get_icons(std::vector<ARGBData>& icons);
		std::string get_title(void);
		bool get_class(std::string& name, std::string& class_name); // WM_CLASS
		bool map(void);
		void unmap(void);
		CARD32 get_desktop(void);
		bool select_input(long event_mask);

		bool set_desktop(CARD32 desktop);
		bool set_decorated(bool decorated);

		bool get_skip_pager(void);

		enum State {
			STICKY = 2,
			SKIP_TASKBAR = 4,
			SKIP_PAGER = 8,
			HIDDEN = 16, // assume this means iconified
			SHADED = 32,
			DEMANDS_ATTENTION = 64
		};

		bool set_state(State state, bool replace = false);
		void get_state(std::vector<Atom> &atoms);
		void get_state(State& state);

		bool iconify(void);

		enum WindowType {
			WINDOW_TYPE_DOCK
		};

		bool set_window_type(WindowType type);

		bool set_background_pixmap(std::tr1::shared_ptr<Pixmap> pixmap);
		bool set_background_colour(CARD32 colour);

		enum StrutType {
			STRUT_LEFT,
			STRUT_RIGHT,
			STRUT_TOP,
			STRUT_BOTTOM
		};

		bool set_partial_strut(StrutType type, CARD32 size, CARD32 start, CARD32 end);
		bool remove_partial_strut(void);

		bool destroy(void);
		void clean_up(::Window drawable); // deletes this and removes from 'windows' map

		// give the windows input focus
		bool activate(void);
		bool clear(void);

		static ::Window get_active_window(void);
	};

}

#endif
